@@ -14,7 +14,6 @@ from enum import Enum
 import bibtexparser
 import dotenv
 import git
-import requests
 import typer
 from git.exc import GitCommandError, InvalidGitRepositoryError, NoSuchPathError
 from typing_extensions import Annotated
@@ -157,10 +156,16 @@ def new_project(
                 repo.git.commit(["-m", "Initialize DVC"])
     ck_info_fpath = os.path.join(abs_path, "calkit.yaml")
     if os.path.isfile(ck_info_fpath) and not overwrite:
-        raise_error(
+        ck_info = calkit.load_calkit_info(wdir=abs_path)
+        name = ck_info.get("name", name)
+        title = ck_info.get("title", title)
+        description = ck_info.get("description", description)
+        typer.echo(
             "Destination is already a Calkit project; "
-            "use --overwrite to continue"
+            "will use existing project info where possible"
         )
+    else:
+        ck_info = {}
     if os.path.isdir(abs_path) and os.listdir(abs_path) and repo is None:
         warn(f"{abs_path} is not empty")
     if name is None:
@@ -224,7 +229,29 @@ def new_project(
                     )
                 repo.git.add("calkit.yaml")
                 if not no_commit:
-                    repo.git.commit(["-m", "Create calkit.yaml"])
+                    repo.git.commit(
+                        ["calkit.yaml", "-m", "Create calkit.yaml"]
+                    )
+            else:
+                # Merge with existing project info in calkit.yaml
+                typer.echo("Updating existing calkit.yaml file")
+                ck_info = (
+                    dict(
+                        owner=resp["owner_account_name"],
+                        name=resp["name"],
+                        title=resp["title"],
+                        description=resp["description"],
+                        git_repo_url=resp["git_repo_url"],
+                    )
+                    | ck_info
+                )
+                with open(calkit_fpath, "w") as f:
+                    ryaml.dump(ck_info, f)
+                repo.git.add("calkit.yaml")
+                if not no_commit and repo.git.diff("--staged"):
+                    repo.git.commit(
+                        ["calkit.yaml", "-m", "Update calkit.yaml"]
+                    )
         try:
             calkit.dvc.configure_remote(wdir=abs_path)
         except Exception:
@@ -283,7 +310,7 @@ def new_project(
         typer.echo("Generating README.md")
         readme_txt = calkit.make_readme_content(
             project_name=name,
-            project_title=title,
+            project_title=title,  # type: ignore
             project_description=description,
         )
         with open(readme_fpath, "w") as f:
@@ -353,7 +380,7 @@ def new_project(
         typer.echo("Generating README.md")
         readme_txt = calkit.make_readme_content(
             project_name=name,
-            project_title=title,
+            project_title=title,  # type: ignore
             project_description=description,
         )
         with open(readme_fpath, "w") as f:
@@ -383,14 +410,14 @@ def new_figure(
     title: Annotated[str, typer.Option("--title")],
     description: Annotated[str, typer.Option("--description")],
     stage_name: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--stage",
             help="Name of the pipeline stage that generates this figure.",
         ),
     ] = None,
     cmd: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--cmd", help="Command to add to the stage, if specified."
         ),
@@ -409,7 +436,7 @@ def new_figure(
         ),
     ] = [],
     outs_from_stage: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--deps-from-stage-outs",
             help="Stage name from which to add outputs as dependencies.",
@@ -588,7 +615,8 @@ def new_docker_env(
     layers: Annotated[
         list[str],
         typer.Option(
-            "--add-layer", help="Add a layer (options: miniforge, foampy)."
+            "--add-layer",
+            help="Add a layer (options: miniforge, foampy, uv, julia).",
         ),
     ] = [],
     env_vars: Annotated[
@@ -703,11 +731,11 @@ def new_docker_env(
         wdir=wdir,
     )
     if base is not None or path is not None:
-        env["path"] = path
+        env["path"] = path  # type: ignore
     if description is not None:
         env["description"] = description
     if layers:
-        env["layers"] = layers
+        env["layers"] = layers  # type: ignore
     if platform:
         env["platform"] = platform
     if user:
@@ -715,18 +743,18 @@ def new_docker_env(
     if gpus:
         env["gpus"] = gpus
     if env_vars:
-        env["env_vars"] = {}
+        env["env_vars"] = {}  # type: ignore
         for var in env_vars:
             if "=" not in var:
                 raise_error(f"Invalid environment variable format: {var}")
             key, value = var.split("=", 1)
             env["env_vars"][key] = value
     if args:
-        env["args"] = args
+        env["args"] = args  # type: ignore
     if deps:
-        env["deps"] = deps
+        env["deps"] = deps  # type: ignore
     if ports:
-        env["ports"] = ports
+        env["ports"] = ports  # type: ignore
     envs[name] = env
     ck_info["environments"] = envs
     with open("calkit.yaml", "w") as f:
@@ -799,14 +827,14 @@ def new_dataset(
     title: Annotated[str, typer.Option("--title")],
     description: Annotated[str, typer.Option("--description")],
     stage_name: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--stage",
             help="Name of the pipeline stage that generates this dataset.",
         ),
     ] = None,
     cmd: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--cmd", help="Command to add to the stage, if specified."
         ),
@@ -825,7 +853,7 @@ def new_dataset(
         ),
     ] = [],
     outs_from_stage: Annotated[
-        str,
+        str | None,
         typer.Option(
             "--deps-from-stage-outs",
             help="Stage name from which to add outputs as dependencies.",
@@ -999,7 +1027,7 @@ def new_publication(
         )
     if template_type is not None:
         try:
-            template_obj = calkit.templates.get_template(template)
+            template_obj = calkit.templates.get_template(template)  # type: ignore
         except ValueError:
             raise_error(f"Template '{template}' does not exist")
     # Parse outs from stage if specified
@@ -1018,7 +1046,8 @@ def new_publication(
     # Create publication object
     if template_type == "latex":
         pub_fpath = os.path.join(
-            path, template_obj.target.removesuffix(".tex") + ".pdf"
+            path,
+            template_obj.target.removesuffix(".tex") + ".pdf",  # type: ignore
         )
     else:
         pub_fpath = path
@@ -1054,7 +1083,7 @@ def new_publication(
         stage = LatexStage(
             kind="latex",
             environment=env_name,
-            target_path=os.path.join(path, template_obj.target),
+            target_path=os.path.join(path, template_obj.target),  # type: ignore
             outputs=[pub_fpath],
         ).model_dump()
         if "pipeline" not in ck_info:
@@ -1066,7 +1095,7 @@ def new_publication(
         calkit.ryaml.dump(ck_info, f)
     repo.git.add("calkit.yaml")
     # Copy in template files if applicable
-    if template_type == "latex":
+    if template is not None and template_type == "latex":
         if overwrite and os.path.exists(path):
             shutil.rmtree(path)
         calkit.templates.use_template(
@@ -1309,7 +1338,7 @@ def new_venv(
         str, typer.Option("--prefix", help="Prefix for environment location.")
     ] = ".venv",
     description: Annotated[
-        str, typer.Option("--description", help="Description.")
+        str | None, typer.Option("--description", help="Description.")
     ] = None,
     overwrite: Annotated[
         bool,
@@ -1390,7 +1419,7 @@ def new_pixi_env(
         list[str], typer.Option("--pip", help="Packages to install with pip.")
     ] = [],
     description: Annotated[
-        str, typer.Option("--description", help="Description.")
+        str | None, typer.Option("--description", help="Description.")
     ] = None,
     platforms: Annotated[
         list[str], typer.Option("--platform", "-p", help="Platform.")
@@ -1479,6 +1508,95 @@ def new_pixi_env(
         repo.git.commit(["-m", f"Add pixi env {name}"])
 
 
+@new_app.command("julia-env")
+def new_julia_env(
+    packages: Annotated[
+        list[str],
+        typer.Argument(help="Packages to include in the environment."),
+    ],
+    name: Annotated[
+        str, typer.Option("--name", "-n", help="Environment name.")
+    ],
+    path: Annotated[
+        str, typer.Option("--path", help="Path for Project.toml file.")
+    ] = "Project.toml",
+    description: Annotated[
+        str | None, typer.Option("--description", help="Description.")
+    ] = None,
+    julia_version: Annotated[
+        str, typer.Option("--julia", "-j", help="Julia version.")
+    ] = "1.11",
+    overwrite: Annotated[
+        bool,
+        typer.Option(
+            "--overwrite",
+            "-f",
+            help="Overwrite any existing environment with this name.",
+        ),
+    ] = False,
+    no_commit: Annotated[
+        bool, typer.Option("--no-commit", help="Do not commit changes.")
+    ] = False,
+):
+    """Create a new Julia environment."""
+    if not os.path.basename(path) == "Project.toml":
+        raise_error(
+            "Julia environment paths must point to a Project.toml file"
+        )
+    try:
+        repo = git.Repo()
+    except git.InvalidGitRepositoryError:
+        raise_error(
+            "Current directory is not a Git repository; run calkit init"
+        )
+    # Add environment to Calkit info
+    ck_info = calkit.load_calkit_info()
+    # If environments is a list instead of a dict, reformulate it
+    envs = ck_info.get("environments", {})
+    if isinstance(envs, list):
+        typer.echo("Converting environments from list to dict")
+        envs = {env.pop("name"): env for env in envs}
+    if name in envs and not overwrite:
+        raise_error(
+            f"Environment with name {name} already exists "
+            "(use -f to overwrite)"
+        )
+    # Create the environment now
+    env_dir = os.path.dirname(path)
+    if env_dir:
+        os.makedirs(env_dir, exist_ok=True)
+    else:
+        env_dir = "."
+    # First make sure the Julia version is installed
+    cmd = ["juliaup", "add", julia_version]
+    try:
+        subprocess.run(cmd, check=True)
+    except subprocess.CalledProcessError:
+        raise_error(f"Failed to install Julia version {julia_version}")
+    cmd = ["julia", f"+{julia_version}", f"--project={env_dir}", "-e"]
+    install_cmd = "using Pkg;"
+    for package in packages:
+        install_cmd += f' Pkg.add("{package}");'
+    cmd.append(install_cmd)
+    try:
+        subprocess.run(cmd, check=True)
+    except subprocess.CalledProcessError:
+        raise_error("Failed to create new Julia environment")
+    typer.echo("Adding environment to calkit.yaml")
+    env = dict(kind="julia", path=path, julia=julia_version)
+    if description is not None:
+        env["description"] = description
+    envs[name] = env
+    ck_info["environments"] = envs
+    with open("calkit.yaml", "w") as f:
+        ryaml.dump(ck_info, f)
+    repo.git.add(path)
+    repo.git.add(os.path.join(env_dir, "Manifest.toml"))
+    repo.git.add("calkit.yaml")
+    if not no_commit and repo.git.diff("--staged"):
+        repo.git.commit(["-m", f"Add Julia env {name}"])
+
+
 class Status(str, Enum):
     in_progress = "in-progress"
     on_hold = "on-hold"
@@ -1786,6 +1904,66 @@ def new_python_script_stage(
     )
 
 
+@new_app.command(name="julia-script-stage")
+def new_julia_script_stage(
+    name: StageArgs.name,
+    environment: StageArgs.environment,
+    script_path: StageArgs.script_path,
+    inputs: StageArgs.inputs = [],
+    outputs: StageArgs.outputs = [],
+    outs_git: StageArgs.outs_git = [],
+    outs_git_no_delete: StageArgs.outs_git_no_delete = [],
+    outs_no_delete: StageArgs.outs_no_delete = [],
+    outs_no_store: StageArgs.outs_no_store = [],
+    outs_no_store_no_delete: StageArgs.outs_no_store_no_delete = [],
+    iter_arg: Annotated[
+        tuple[str, str] | None,
+        typer.Option(
+            "--iter",
+            help=(
+                "Iterate over an argument with a comma-separated list, e.g., "
+                "--iter-arg var_name val1,val2,val3."
+            ),
+        ),
+    ] = None,
+    overwrite: StageArgs.overwrite = False,
+    no_check: StageArgs.no_check = False,
+    no_commit: StageArgs.no_commit = False,
+) -> None:
+    """Add a stage to the pipeline that runs a Julia script."""
+    ck_outs = _to_ck_outs(
+        outputs=outputs,
+        outs_git=outs_git,
+        outs_git_no_delete=outs_git_no_delete,
+        outs_no_delete=outs_no_delete,
+        outs_no_store=outs_no_store,
+        outs_no_store_no_delete=outs_no_store_no_delete,
+    )
+    try:
+        if iter_arg is not None:
+            arg_name, vals = iter_arg
+            i = [StageIteration(arg_name=arg_name, values=vals.split(","))]  # type: ignore
+        else:
+            i = None
+        stage = calkit.models.pipeline.JuliaScriptStage(
+            kind="julia-script",
+            environment=environment,
+            inputs=inputs,  # type: ignore
+            outputs=ck_outs,
+            script_path=script_path,
+            iterate_over=i,
+        )
+    except Exception as e:
+        raise_error(f"Invalid stage specification: {e}")
+    _save_stage(
+        stage=stage,
+        name=name,
+        overwrite=overwrite,
+        no_check=no_check,
+        no_commit=no_commit,
+    )
+
+
 @new_app.command(name="matlab-script-stage")
 def new_matlab_script_stage(
     name: StageArgs.name,
@@ -1815,7 +1993,7 @@ def new_matlab_script_stage(
         stage = calkit.models.pipeline.MatlabScriptStage(
             kind="matlab-script",
             environment=environment,
-            inputs=inputs,
+            inputs=inputs,  # type: ignore
             outputs=ck_outs,
             script_path=script_path,
         )
@@ -1862,7 +2040,7 @@ def new_latex_stage(
             kind="latex",
             environment=environment,
             target_path=target_path,
-            inputs=inputs,
+            inputs=inputs,  # type: ignore
             outputs=ck_outs,
         )
     except Exception as e:
@@ -1966,7 +2144,8 @@ def new_release(
             "--name",
             "-n",
             help=(
-                "A name for the release, typically kebab-case. "
+                "A name for the release, typically kebab-case or a semantic "
+                "version. "
                 "Will be used for the Git tag and GitHub release title."
             ),
         ),
@@ -1978,8 +2157,8 @@ def new_release(
         str,
         typer.Argument(help="The path to release; '.' for a project release."),
     ] = ".",
-    description: Annotated[
-        str,
+    release_description: Annotated[
+        str | None,
         typer.Option(
             "--description",
             "--desc",
@@ -1990,7 +2169,7 @@ def new_release(
         ),
     ] = None,
     release_date: Annotated[
-        str,
+        str | None,
         typer.Option("--date", help="Release date. Will default to today."),
     ] = None,
     dry_run: Annotated[
@@ -2014,8 +2193,54 @@ def new_release(
             help="Do not push to Git remote.",
         ),
     ] = False,
+    no_github_release: Annotated[
+        bool,
+        typer.Option(
+            "--no-github",
+            help="Do not create a GitHub release.",
+        ),
+    ] = False,
+    to: Annotated[
+        str,
+        typer.Option(
+            "--to", help="Archival service to use (zenodo or caltechdata)."
+        ),
+    ] = "zenodo",
+    draft_only: Annotated[
+        bool,
+        typer.Option(
+            "--draft",
+            help="Create draft record with reserved DOI but do not publish.",
+        ),
+    ] = False,
+    license_ids: Annotated[
+        list[str],
+        typer.Option(
+            "--license",
+            help=(
+                "License ID (from https://spdx.org/licenses). "
+                "Multiple can be specified. "
+                "Will try to infer from LICENSE file, if present."
+            ),
+        ),
+    ] = [],
+    verbose: Annotated[
+        bool,
+        typer.Option(
+            "--verbose",
+            "-v",
+            help="Print verbose output.",
+        ),
+    ] = False,
 ):
     """Create a new release."""
+    to = to.lower()
+    if to not in calkit.releases.SERVICES:
+        raise_error(
+            f"Unknown archival service '{to}'; "
+            f"choose from: {list(calkit.releases.SERVICES.keys())}"
+        )
+    publisher_name = calkit.releases.SERVICES[to]["name"]
     if release_type not in [
         "project",
         "publication",
@@ -2026,12 +2251,12 @@ def new_release(
         raise_error(f"Unknown release type '{release_type}'")
     # TODO: Check path is consistent with release type
     dotenv.load_dotenv()
-    # First see if we have a Zenodo token
-    typer.echo("Checking for Zenodo token")
+    # Check that we have access to the service
+    typer.echo(f"Checking for {to} token")
     try:
-        token = calkit.zenodo.get_token()
+        calkit.invenio.get_token(service=to)  # type: ignore
     except Exception as e:
-        raise_error(e)
+        raise_error(str(e))
     ck_info = calkit.load_calkit_info()
     releases = ck_info.get("releases", {})
     # TODO: Enable resuming a release if upload failed part-way?
@@ -2053,6 +2278,7 @@ def new_release(
     if release_date is None:
         release_date = str(calkit.utcnow().date())
     typer.echo(f"Using release date: {release_date}")
+    project_description = ck_info.get("description")
     # Gather up the list of files to upload
     if path == ".":
         zip_path = release_files_dir + "/archive.zip"
@@ -2061,21 +2287,20 @@ def new_release(
         with zipfile.ZipFile(zip_path, "w") as zipf:
             for fpath in all_paths:
                 zipf.write(fpath)
-        if description is None:
-            description = "An archive of all project files."
         title = ck_info.get("title")
         if title is None:
             warn("Project has no title")
             title = typer.prompt("Enter a title for the project")
             ck_info["title"] = title
+            if not dry_run:
+                with open("calkit.yaml", "w") as f:
+                    calkit.ryaml.dump(ck_info, f)
     else:
         # TODO: Handle directories, e.g., datasets
         if not os.path.isfile(path):
             raise_error("Single artifact releases must be a single file")
         typer.echo(f"Copying {path} into {release_files_dir}")
         shutil.copy2(path, release_files_dir)
-        if description is None:
-            description = f"Release {release_type} at {path}."
         # Check that this artifact actually exists
         artifact_key = (
             release_type + "s" if release_type != "software" else release_type
@@ -2117,18 +2342,91 @@ def new_release(
     size = calkit.get_size(release_files_dir)
     typer.echo(f"Release size: {(size / 1e6):.1f} MB")
     if size >= 50e9:
-        raise_error("Release is too large (>50 GB) to upload to Zenodo")
-    # Upload to Zenodo
-    # Is there already a deposition for this release, which indicates we should
+        raise_error(f"Release is too large (>50 GB) to upload to {to}")
+    # Upload to InvenioRDM instance
+    # Is there already a record for this release, which indicates we should
     # create a new version?
-    zenodo_dep_id = None
-    project_name = calkit.detect_project_name()
-    zenodo_metadata = dict(
+    try:
+        project_name = calkit.detect_project_name(prepend_owner=True)
+    except Exception:
+        raise_error("Could not detect project name")
+    invenio_description = ""
+    if project_description is not None:
+        invenio_description += project_description + "\n\n"
+    if release_description is not None:
+        invenio_description += release_description + "\n\n"
+    invenio_description += (
+        f"This is an archive of release '{name}' from the Calkit project "
+        f"'{project_name}'."
+    )
+    record_id = None
+    # Detect project license IDs if necessary
+    if not license_ids:
+        license_file = None
+        license_txt = None
+        license_candidates = [
+            "LICENSE",
+            "LICENSE.rst",
+            "LICENSE.txt",
+            "LICENSE.md",
+        ]
+        for lc in license_candidates:
+            if os.path.isfile(lc):
+                license_file = lc
+                break
+        if license_file is None:
+            typer.echo("No project license found.")
+            use_default_license = typer.confirm(
+                "Would you like to use the default (MIT/CC-BY-4.0)?",
+                default=True,
+            )
+            if not use_default_license:
+                raise_error("Please generate a license file and try again")
+            copyright_holder = typer.prompt(
+                "Please enter the copyright holder for the license "
+                "(e.g., your full name)"
+            )
+            license_txt = calkit.licenses.LICENSE_TEMPLATE_DUAL.format(
+                year=calkit.utcnow().year, copyright_holder=copyright_holder
+            )
+            with open("LICENSE", "w") as f:
+                f.write(license_txt)
+                repo.git.add("LICENSE")
+            license_ids = ["mit", "cc-by-4.0"]
+        if license_txt is None and license_file is not None:
+            with open(license_file) as f:
+                license_txt = f.read()
+            # Try to detect licenses
+            if "the mit license" in license_txt.lower():
+                license_ids.append("mit")
+            if "cc by 4.0" in license_txt.lower():
+                license_ids.append("cc-by-4.0")
+            # TODO: Detect others
+    if not license_ids:
+        raise_error("Project has no license(s) defined")
+    invenio_metadata = dict(
         title=title,
-        description=description,
+        description=invenio_description,
         notes=f"Created from Calkit project {project_name} release {name}.",
         publication_date=release_date,
+        version=name,
+        publisher=publisher_name,
+        rights=[{"id": lid for lid in license_ids}],
     )
+    # Add related identifiers
+    github_url = calkit.detect_project_github_url()
+    related_identifiers = []
+    if github_url is not None:
+        related_identifiers.append(
+            {
+                "identifier": github_url,
+                "scheme": "url",
+                "relation_type": {"id": "issupplementto"},
+                "resource_type": {"id": "other"},
+            }
+        )
+    invenio_metadata["related_identifiers"] = related_identifiers
+    # TODO: Add calkit.io URL if applicable?
     # Determine creators from authors, adding to project if not present
     authors = ck_info.get("authors", [])
     if not authors:
@@ -2147,10 +2445,10 @@ def new_release(
             author["affiliation"] = typer.prompt(
                 f"Enter the affiliation of author {n}"
             )
-            has_orchid = typer.confirm(
+            has_orcid = typer.confirm(
                 f"Does author {n} have an ORCID?", default=False
             )
-            if has_orchid:
+            if has_orcid:
                 author["orcid"] = typer.prompt(
                     f"Enter the ORCID of author {n}"
                 )
@@ -2159,112 +2457,151 @@ def new_release(
                 "Are there more authors to enter?", default=True
             )
         ck_info["authors"] = authors
-    zenodo_creators = []
+        # Write authors out to calkit.yaml
+        if not dry_run:
+            typer.echo("Adding authors to calkit.yaml")
+            with open("calkit.yaml", "w") as f:
+                calkit.ryaml.dump(ck_info, f)
+    invenio_creators = []
     for author in authors:
-        creator = dict(
-            name=f"{author['last_name']}, {author['first_name']}",
-            affiliation=author["affiliation"],
-        )
-        if "orcid" in author:
-            creator["orcid"] = author["orcid"]
-        zenodo_creators.append(creator)
-    zenodo_metadata["creators"] = zenodo_creators
-    if release_type == "project":
-        zenodo_metadata["upload_type"] = "other"
-    elif release_type == "publication":
-        pubtype = artifact.get("kind")
+        orcid = author.get("orcid")
+        creator = {
+            "person_or_org": {
+                "type": "personal",
+                "given_name": author["first_name"],
+                "family_name": author["last_name"],
+                "identifiers": [{"identifier": orcid}] if orcid else [],
+            },
+            "affiliations": [{"name": author["affiliation"]}],
+        }
+        invenio_creators.append(creator)
+    invenio_metadata["creators"] = invenio_creators  # type: ignore
+    # Set InvenioRDM resource_type based on release_type and artifact kind
+    if release_type == "publication":
+        pubtype = artifact.get("kind")  # type: ignore
         if pubtype == "journal-article":
-            zenodo_metadata["upload_type"] = "publication"
-            zenodo_metadata["publication_type"] = "article"
+            resource_type = "publication-article"
         elif pubtype == "presentation":
-            zenodo_metadata["upload_type"] = "presentation"
+            resource_type = "presentation"
         elif pubtype == "poster":
-            zenodo_metadata["upload_type"] = "poster"
+            resource_type = "poster"
         else:
-            zenodo_metadata["upload_type"] = "other"
-    elif release_type in ["dataset", "software"]:
-        zenodo_metadata["upload_type"] = release_type
+            resource_type = "publication-other"
     elif release_type == "figure":
-        zenodo_metadata["upload_type"] = "image"
-        zenodo_metadata["image_type"] = "figure"
+        resource_type = "image-figure"
+    elif release_type in ["dataset", "software", "poster", "presentation"]:
+        resource_type = release_type
     else:
-        zenodo_metadata["upload_type"] = "other"
+        # Default for "project" and other unknown types
+        resource_type = "other"
+    invenio_metadata["resource_type"] = {"id": resource_type}  # type: ignore
     doi = None
     url = None
     for existing_name, existing_release in releases.items():
         if (
             existing_release.get("kind") == release_type
             and existing_release.get("path") == path
-            and existing_release.get("publisher") == "zenodo.org"
+            and existing_release.get("publisher") == to
         ):
-            zenodo_dep_id = existing_release.get("zenodo_dep_id")
+            record_id = existing_release.get("record_id")
             typer.echo(
-                f"Found existing Zenodo deposition ID {zenodo_dep_id} "
+                f"Found existing {to} record ID {record_id} "
                 f"in release {existing_name} to create new version for"
             )
             break
     if not dry_run:
-        typer.echo("Uploading to Zenodo")
-        if zenodo_dep_id is not None:
-            # Create a new version of the existing deposit
+        typer.echo(f"Uploading to {to}")
+        if record_id is not None:
+            # Create a new version of the existing record
             # TODO: This might fail if a new version is in progress, in which
             # case we should discard that
-            zenodo_dep = calkit.zenodo.post(
-                f"/deposit/depositions/{zenodo_dep_id}/actions/newversion",
-                json=dict(metadata=zenodo_metadata),
-            )
-            typer.echo("Created new version deposition")
-            typer.echo("Fetching latest draft")
-            zenodo_dep = requests.get(
-                zenodo_dep["links"]["latest_draft"],
-                params=dict(access_token=token),
-            ).json()
-            zenodo_dep_id = zenodo_dep["id"]
-            typer.echo(
-                f"Fetched latest draft with deposition ID: {zenodo_dep_id} "
+            invenio_dep = calkit.invenio.post(
+                f"/records/{record_id}/versions",
+                service=to,  # type: ignore
             )
+            typer.echo("Created new version record")
+            record_id = invenio_dep["id"]
+            typer.echo(f"Created new version with record ID: {record_id}")
             # Now update that draft with the metadata
-            typer.echo("Updating latest draft metadata")
-            calkit.zenodo.put(
-                f"/deposit/depositions/{zenodo_dep_id}",
-                json=dict(metadata=zenodo_metadata),
+            typer.echo("Updating draft metadata")
+            calkit.invenio.put(
+                f"/records/{record_id}/draft",
+                json=dict(metadata=invenio_metadata),
+                service=to,  # type: ignore
             )
         else:
-            zenodo_dep = calkit.zenodo.post(
-                "/deposit/depositions", json=dict(metadata=zenodo_metadata)
+            invenio_dep = calkit.invenio.post(
+                "/records",
+                json=dict(metadata=invenio_metadata),
+                service=to,  # type: ignore
             )
-            zenodo_dep_id = zenodo_dep["id"]
-        bucket_url = zenodo_dep["links"]["bucket"]
+            if verbose:
+                typer.echo(f"Invenio records post response:\n{invenio_dep}")
+            record_id = invenio_dep["id"]
         files = os.listdir(release_files_dir)
         for filename in files:
             typer.echo(f"Uploading {filename}")
             fpath = os.path.join(release_files_dir, filename)
+            # First, initiate the file upload
+            calkit.invenio.post(
+                f"/records/{record_id}/draft/files",
+                json=[{"key": filename}],
+                service=to,  # type: ignore
+            )
+            # Then upload the file content
             with open(fpath, "rb") as f:
-                resp = requests.put(
-                    f"{bucket_url}/{filename}",
-                    data=f,
-                    params={"access_token": token},
+                file_data = f.read()
+                resp = calkit.invenio.put(
+                    f"/records/{record_id}/draft/files/{filename}/content",
+                    headers={"Content-Type": "application/octet-stream"},
+                    as_json=False,
+                    service=to,  # type: ignore
+                    data=file_data,
                 )
                 typer.echo(f"Status code: {resp.status_code}")
-                resp.raise_for_status()
-        # Now publish the new deposition
-        typer.echo(f"Publishing Zenodo deposition ID {zenodo_dep_id}")
-        zenodo_dep = calkit.zenodo.post(
-            f"/deposit/depositions/{zenodo_dep_id}/actions/publish"
-        )
-        zenodo_dep_id = zenodo_dep["id"]
-        doi = zenodo_dep["doi"]
-        url = zenodo_dep["doi_url"]
-        typer.echo(f"Published to Zenodo with DOI: {doi}")
+            # Commit the file
+            calkit.invenio.post(
+                f"/records/{record_id}/draft/files/{filename}/commit",
+                service=to,  # type: ignore
+            )
+        # Conditionally publish or reserve DOI based on --draft flag
+        if draft_only:
+            # Reserve a DOI for the draft record
+            typer.echo(f"Reserving DOI for {to} draft record ID {record_id}")
+            doi_resp = calkit.invenio.post(
+                f"/records/{record_id}/draft/pids/doi",
+                service=to,  # type: ignore
+            )
+            if verbose:
+                typer.echo(f"DOI response for draft:\n{doi_resp}")
+            try:
+                doi = doi_resp["pids"]["doi"]["identifier"]
+            except KeyError:
+                doi = doi_resp["doi"]
+            except Exception as e:
+                raise_error(f"Failed to reserve DOI for draft: {e}")
+            url = f"https://doi.org/{doi}"
+            typer.echo(f"Created {to} draft with reserved DOI: {doi}")
+        else:
+            # Publish the record
+            typer.echo(f"Publishing {to} record ID {record_id}")
+            invenio_dep = calkit.invenio.post(
+                f"/records/{record_id}/draft/actions/publish",
+                service=to,  # type: ignore
+            )
+            record_id = invenio_dep["id"]
+            doi = invenio_dep["pids"]["doi"]["identifier"]
+            url = f"https://doi.org/{doi}"
+            typer.echo(f"Published to {to} with DOI: {doi}")
     else:
-        typer.echo(f"Would have posted Zenodo deposition: {zenodo_metadata}")
-    # If this is a project release, add Zenodo badge to project README if
-    # it doesn't exist
+        typer.echo(f"Would have posted {to} record: {invenio_metadata}")
+    # If this is a project release, add badge to project README
     doi_md = None
     if release_type == "project" and doi is not None:
         typer.echo("Adding DOI badge to README.md")
+        doi_base_url = calkit.releases.SERVICES[to]["url"]
         doi_md = (
-            f"[![DOI](https://zenodo.org/badge/DOI/{doi}.svg)]"
+            f"[![DOI]({doi_base_url}/badge/DOI/{doi}.svg)]"
             f"(https://handle.stage.datacite.org/{doi})"
         )
         if os.path.isfile("README.md"):
@@ -2298,11 +2635,15 @@ def new_release(
         if not dry_run:
             repo.git.add("README.md")
     # Create Git tag
-    if not dry_run:
-        repo.git.tag(["-a", name, "-m", description])
+    git_tag_message = release_description
+    if git_tag_message is None:
+        git_tag_message = f"Release {name}"
+    if not dry_run and not draft_only:
+        repo.git.tag(["-a", name, "-m", git_tag_message])
     else:
         typer.echo(
-            f"Would have created Git tag {name} with message: {description}"
+            f"Would have created Git tag {name} with message: "
+            f"{git_tag_message}"
         )
     # Save release in Calkit info
     release = dict(
@@ -2310,11 +2651,11 @@ def new_release(
         path=path,
         git_rev=git_rev,
         date=release_date,
-        publisher="zenodo.org",
-        zenodo_dep_id=zenodo_dep_id,
+        publisher=to,
+        record_id=record_id,
         doi=doi,
         url=url,
-        description=description,
+        description=release_description,
     )
     releases[name] = release
     ck_info["releases"] = releases
@@ -2345,14 +2686,15 @@ def new_release(
                 reflib = bibtexparser.load(f)
         else:
             reflib = bibtexparser.bibdatabase.BibDatabase()
-        zenodo_bibtex = calkit.releases.create_bibtex(
+        invenio_bibtex = calkit.releases.create_bibtex(
             authors=authors,
             release_date=release_date,
-            title=title,
-            doi=doi,
-            dep_id=zenodo_dep_id,
+            title=title,  # type: ignore
+            doi=doi,  # type: ignore
+            record_id=record_id,  # type: ignore
+            service=to,  # type: ignore
         )
-        new_entry = bibtexparser.loads(zenodo_bibtex).entries[0]
+        new_entry = bibtexparser.loads(invenio_bibtex).entries[0]
         # Search through entries for one with the same DOI, and replace if
         # there is a match
         existing_index = None
@@ -2381,24 +2723,27 @@ def new_release(
     if not dry_run and calkit.git.get_staged_files() and not no_commit:
         repo.git.commit(["-m", f"Create new {release_type} release {name}"])
     # Push with Git
-    if not dry_run and not no_push and not no_commit:
+    if not dry_run and not no_push and not no_commit and not draft_only:
         repo.git.push(["origin", repo.active_branch.name, "--tags"])
         # Now create GitHub release
-        typer.echo("Creating GitHub release")
-        release_body = ""
-        if doi_md is not None:
-            release_body += doi_md + "\n\n"
-        release_body += description
-        resp = calkit.cloud.post(
-            f"/projects/{project_name}/github-releases",
-            json=dict(
-                tag_name=name,
-                body=release_body,
-            ),
-        )
-        typer.echo(f"Created GitHub release at: {resp['url']}")
-        # TODO: Upload assets for GitHub release if they're not too big?
+        if not no_github_release:
+            typer.echo("Creating GitHub release")
+            release_body = ""
+            if doi_md is not None:
+                release_body += doi_md + "\n\n"
+            if release_description is not None:
+                release_body += release_description
+            resp = calkit.cloud.post(
+                f"/projects/{project_name}/github-releases",
+                json=dict(
+                    tag_name=name,
+                    body=release_body,
+                ),
+            )
+            typer.echo(f"Created GitHub release at: {resp['url']}")
+            # TODO: Upload assets for GitHub release if they're not too big?
     typer.echo(f"New {release_type} release {name} successfully created")
+    return release
 
 
 @new_app.command(name="stage")
