@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 import filecmp
+import json
 import os
 import shutil
 import subprocess
@@ -36,17 +37,6 @@ def import_publication(
             help="Directory at which to save in the project, e.g., 'paper'."
         ),
     ],
-    sync_paths: Annotated[
-        list[str],
-        typer.Option(
-            "--sync-path",
-            "-s",
-            help=(
-                "Paths to sync from the Overleaf project, e.g., 'main.tex'. "
-                "Note that multiple can be specified."
-            ),
-        ),
-    ],
     title: Annotated[
         str,
         typer.Option(
@@ -55,6 +45,14 @@ def import_publication(
             help="Title of the publication.",
         ),
     ],
+    target_path: Annotated[
+        str | None,
+        typer.Option(
+            "--target",
+            "-T",
+            help="Target TeX file path inside Overleaf project.",
+        ),
+    ] = None,
     description: Annotated[
         str | None,
         typer.Option(
@@ -70,6 +68,17 @@ def import_publication(
             help="What of the publication this is, e.g., 'journal-article'.",
         ),
     ] = None,
+    sync_paths: Annotated[
+        list[str],
+        typer.Option(
+            "--sync-path",
+            "-s",
+            help=(
+                "Paths to sync from the Overleaf project, e.g., 'main.tex'. "
+                "Note that multiple can be specified."
+            ),
+        ),
+    ] = [],
     push_paths: Annotated[
         list[str],
         typer.Option(
@@ -82,18 +91,6 @@ def import_publication(
             ),
         ),
     ] = [],
-    pdf_path: Annotated[
-        str | None,
-        typer.Option(
-            "--pdf-path",
-            "-o",
-            help=(
-                "PDF output file in the Overleaf project, e.g., 'main.pdf'. "
-                "If not provided, it will be determined from the first sync "
-                "path."
-            ),
-        ),
-    ] = None,
     no_commit: Annotated[
         bool,
         typer.Option("--no-commit", help="Do not commit changes to repo."),
@@ -127,31 +124,26 @@ def import_publication(
         typer.echo("Storing Overleaf token in Calkit config")
         config.overleaf_token = overleaf_token
         config.write()
-    if not src_url.startswith("https://www.overleaf.com/project/"):
+    if (
+        not src_url.startswith("https://www.overleaf.com/project/")
+        and calkit.config.get_env() != "test"
+    ):
         raise_error(
             "Invalid URL; must start with 'https://www.overleaf.com/project/'"
         )
     overleaf_project_id = src_url.split("/")[-1]
     if not overleaf_project_id:
         raise_error("Invalid Overleaf project ID")
+    # Check target path
+    if target_path is not None and not target_path.endswith(".tex"):
+        raise_error("Target path should have a .tex extension")
+    # Make sure destination directory exists, and isn't a file
+    if os.path.isfile(dest_dir):
+        raise_error("Destination must be a directory, not a file")
+    os.makedirs(dest_dir, exist_ok=True)
     ck_info = calkit.load_calkit_info(process_includes="environments")
     pubs = ck_info.get("publications", [])
     # TODO: Don't allow the same Overleaf project ID in multiple publications
-    # Determine the PDF output path
-    if pdf_path is None:
-        # Use the first sync path as the PDF path
-        pdf_path = sync_paths[0].removesuffix(".tex") + ".pdf"
-        typer.echo(f"Using PDF path: {pdf_path}")
-    tex_path = pdf_path.removesuffix(".pdf") + ".tex"
-    pub_path = PurePosixPath(dest_dir, pdf_path).as_posix()
-    pub_paths = [pub.get("path") for pub in pubs]
-    if not overwrite and pub_path in pub_paths:
-        raise_error(
-            f"A publication already exists in this project at {pub_path}"
-        )
-    elif overwrite and pub_path in pub_paths:
-        # Note: This publication will go to the end of the list
-        pubs = [p for p in pubs if p.get("path") != pub_path]
     repo = git.Repo()
     # Clone the Overleaf project into .calkit/overleaf if it doesn't exist
     # otherwise pull
@@ -159,8 +151,8 @@ def import_publication(
     os.makedirs(overleaf_dir, exist_ok=True)
     git_ignore(overleaf_dir, no_commit=no_commit)
     overleaf_project_dir = os.path.join(overleaf_dir, overleaf_project_id)
-    git_clone_url = (
-        f"https://git:{overleaf_token}@git.overleaf.com/{overleaf_project_id}"
+    git_clone_url = calkit.overleaf.get_git_remote_url(
+        project_id=overleaf_project_id, token=overleaf_token
     )
     if os.path.isdir(overleaf_project_dir):
         warn("This Overleaf project has already been cloned; removing")
@@ -172,6 +164,36 @@ def import_publication(
         overleaf_project_dir,
         depth=1,
     )
+    # Detect target path if not specified
+    if target_path is None:
+        ol_contents = os.listdir(overleaf_project_dir)
+        for cand in ["main.tex", "report.tex", "paper.tex"]:
+            if cand in ol_contents:
+                target_path = cand
+                break
+    if target_path is None:
+        # Fall back to lone .tex file if there is one
+        tex_files = [p for p in ol_contents if p.endswith(".tex")]
+        if len(tex_files) == 1:
+            target_path = tex_files[0]
+    if target_path is None:
+        raise_error(
+            "Target TeX file path cannot be detected; "
+            "please specify with --target"
+        )
+    # Determine the PDF output path
+    pdf_path = target_path.removesuffix(".tex") + ".pdf"  # type: ignore
+    typer.echo(f"Using PDF path: {pdf_path}")
+    tex_path = pdf_path.removesuffix(".pdf") + ".tex"
+    pub_path = PurePosixPath(dest_dir, pdf_path).as_posix()
+    pub_paths = [pub.get("path") for pub in pubs]
+    if not overwrite and pub_path in pub_paths:
+        raise_error(
+            f"A publication already exists in this project at {pub_path}"
+        )
+    elif overwrite and pub_path in pub_paths:
+        # Note: This publication will go to the end of the list
+        pubs = [p for p in pubs if p.get("path") != pub_path]
     # Check that we have a LaTeX environment
     typer.echo("Checking that this project has a LaTeX environment")
     envs = ck_info.get("environments", {})
@@ -193,6 +215,8 @@ def import_publication(
             description="TeXlive via Docker.",
         )
         ck_info["environments"] = envs
+        with open("calkit.yaml", "w") as f:
+            calkit.ryaml.dump(ck_info, f)
     # Check that we have a build stage
     # TODO: Use Calkit pipeline for this
     typer.echo("Checking for a build stage in the pipeline")
@@ -220,7 +244,6 @@ def import_publication(
             name=stage_name,
             environment=tex_env_name,
             target_path=PurePosixPath(dest_dir, tex_path).as_posix(),
-            outputs=[pub_path],
             inputs=[
                 os.path.join(dest_dir, p) for p in sync_paths + push_paths
             ],
@@ -230,6 +253,7 @@ def import_publication(
         repo.git.add("calkit.yaml")
     # Add to publications in calkit.yaml
     typer.echo("Adding publication to calkit.yaml")
+    ck_info = calkit.load_calkit_info()
     new_pub = dict(
         path=pub_path,
         title=title,
@@ -249,11 +273,15 @@ def import_publication(
     with open("calkit.yaml", "w") as f:
         calkit.ryaml.dump(ck_info, f)
     repo.git.add("calkit.yaml")
-    if not no_commit:
+    if not no_commit and repo.git.diff(["--staged", "--", "calkit.yaml"]):
         # Commit any necessary changes
         typer.echo("Committing changes")
         repo.git.commit(
-            ["-m", f"Import Overleaf project ID {overleaf_project_id}"]
+            [
+                "calkit.yaml",
+                "-m",
+                f"Import Overleaf project ID {overleaf_project_id}",
+            ]
         )
     # Sync the project
     sync(paths=[pub_path], no_commit=no_commit)
@@ -297,12 +325,12 @@ def sync(
             help="Enable verbose output.",
         ),
     ] = False,
-    force: Annotated[
+    resolve: Annotated[
         bool,
         typer.Option(
-            "--force",
-            "-f",
-            help="Force push to Overleaf even if there are conflicts.",
+            "--resolve",
+            "-r",
+            help="Mark merge conflicts as resolved before committing.",
         ),
     ] = False,
 ):
@@ -319,11 +347,33 @@ def sync(
     calkit_config = calkit.config.read()
     overleaf_token = calkit_config.overleaf_token
     if not overleaf_token:
-        raise_error(
-            "Overleaf token not set; "
-            "Please set it using 'calkit config set overleaf_token'"
-        )
+        # See if we can get it from the cloud
+        if calkit_config.token is not None:
+            try:
+                resp = calkit.cloud.get("/user/overleaf-token")
+                overleaf_token = resp["access_token"]
+                calkit_config.overleaf_token = overleaf_token
+                calkit_config.write()
+            except Exception:
+                raise_error(
+                    "Overleaf token not set; "
+                    "Please set it using 'calkit config set overleaf_token'"
+                )
     repo = git.Repo()
+    conflict_fpath = os.path.join(".calkit", "overleaf", "CONFLICT.json")
+    in_am_session = "in the middle of an am session" in repo.git.status()
+    # Check if we're in the middle of resolving a merge conflict
+    if in_am_session and not resolve:
+        raise_error(
+            "You are in the middle of resolving a merge conflict; "
+            "use 'calkit overleaf sync --resolve' after editing file(s)"
+        )
+    elif resolve:
+        if not os.path.isfile(conflict_fpath):
+            raise_error("No merge conflict to resolve")
+        # Figure out which wdir has the conflict in it
+        with open(conflict_fpath) as f:
+            resolving_info = json.load(f)
     for pub in pubs:
         overleaf_config = pub.get("overleaf", {})
         if not overleaf_config:
@@ -346,6 +396,16 @@ def sync(
                 "No working directory defined for this publication; "
                 "please set it in the publication's Overleaf config"
             )
+        if resolve and wdir == resolving_info["wdir"]:
+            repo.git.add(wdir)
+            if repo.git.diff(["--staged", wdir]):
+                repo.git.commit(
+                    [wdir, "-m", f"Resolve Overleaf merge conflict in {wdir}"]
+                )
+            if in_am_session:
+                repo.git.am("--skip")
+        elif resolve:
+            continue
         # If there are any uncommitted changes in the publication working
         # directory, raise an error
         if repo.git.diff(wdir) or repo.index.diff("HEAD", wdir):
@@ -357,9 +417,8 @@ def sync(
         overleaf_project_dir = os.path.join(
             ".calkit", "overleaf", overleaf_project_id
         )
-        overleaf_remote_url = (
-            f"https://git:{overleaf_token}@git.overleaf.com/"
-            f"{overleaf_project_id}"
+        overleaf_remote_url = calkit.overleaf.get_git_remote_url(
+            project_id=overleaf_project_id, token=str(overleaf_token)
         )
         if not os.path.isdir(overleaf_project_dir):
             overleaf_repo = git.Repo.clone_from(
@@ -381,7 +440,10 @@ def sync(
                 "it matches one in your Overleaf account settings "
                 "(https://overleaf.com/user/settings)"
             )
-        last_sync_commit = pub["overleaf"].get("last_sync_commit")
+        if resolve:
+            last_sync_commit = resolving_info["last_overleaf_commit"]
+        else:
+            last_sync_commit = pub["overleaf"].get("last_sync_commit")
         if last_sync_commit:
             commits_since = list(
                 overleaf_repo.iter_commits(rev=f"{last_sync_commit}..HEAD")
@@ -390,6 +452,8 @@ def sync(
                 f"There have been {len(commits_since)} changes on "
                 "Overleaf since last sync"
             )
+        else:
+            commits_since = []
         # Determine which paths to sync and push
         # TODO: Support glob patterns
         git_sync_paths = pub["overleaf"].get("sync_paths", [])
@@ -397,6 +461,14 @@ def sync(
         dvc_sync_paths = pub["overleaf"].get("dvc_sync_paths", [])
         sync_paths = git_sync_paths + dvc_sync_paths
         push_paths = pub["overleaf"].get("push_paths", [])
+        implicit_sync_paths = os.listdir(overleaf_repo.working_dir)
+        for p in implicit_sync_paths:
+            if p.startswith("."):
+                continue
+            if p not in sync_paths:
+                sync_paths.append(p)
+                if p not in git_sync_paths and p not in dvc_sync_paths:
+                    git_sync_paths.append(p)
         git_sync_paths_in_project = [
             os.path.join(wdir, p) for p in git_sync_paths
         ]
@@ -431,7 +503,6 @@ def sync(
                         "git",
                         "am",
                         "--3way",
-                        "--reject",
                         "--directory",
                         wdir,
                         "-",
@@ -439,25 +510,38 @@ def sync(
                     input=patch,
                     text=True,
                     encoding="utf-8",
+                    capture_output=True,
                 )
-                if process.returncode != 0:
-                    if force:
-                        # Skip any patches we might be in the middle of
-                        try:
-                            repo.git.am("--skip")
-                        except Exception:
-                            pass
-                        typer.echo(
-                            "Failed to apply Overleaf patch to project repo. "
-                            "Proceeding to push project changes to Overleaf "
-                            "anyway."
-                        )
-                    else:
-                        raise_error(
-                            "Failed to apply Overleaf patch to project repo. "
-                            "Check the .rej files and manually apply changes, "
-                            "delete them, then rerun with --force."
+                # Handle merge conflicts
+                if (
+                    process.returncode != 0
+                    and "merge conflict" in process.stdout.lower()
+                ):
+                    msg = ""
+                    for line in process.stdout.split("\n"):
+                        if "merge conflict" in line.lower():
+                            msg += line + "\n"
+                    # Save a file to track this merge conflict
+                    c = overleaf_repo.head.commit.hexsha
+                    with open(conflict_fpath, "w") as f:
+                        json.dump(
+                            {
+                                "wdir": wdir,
+                                "last_overleaf_commit": c,
+                                "pub_path": pub["path"],
+                            },
+                            f,
                         )
+                    raise_error(
+                        f"{msg}Edit the file(s) and then call:\n\n"
+                        "    calkit overleaf sync --resolve"
+                    )
+                elif process.returncode != 0:
+                    raise_error(f"Could not apply:\n{process.stdout}")
+            elif resolve:
+                # We have no patch since the last sync, but we need to update
+                # our latest sync commit
+                typer.echo("Merge conflict resolved")
             else:
                 typer.echo("No changes to apply")
         else:
@@ -527,6 +611,8 @@ def sync(
         with open("calkit.yaml", "w") as f:
             calkit.ryaml.dump(ck_info, f)
         repo.git.add("calkit.yaml")
+        if resolve and os.path.isfile(conflict_fpath):
+            os.remove(conflict_fpath)
         # Stage the changes in the project repo
         # Add any DVC sync paths to DVC
         for dvc_sync_path in dvc_sync_paths_in_project:
@@ -560,6 +646,8 @@ def sync(
                 commit_message,
             )
     if not no_push and not no_commit:
+        if not repo.remotes:
+            raise_error("Project has no Git remotes defined")
         # Push to the project remote
         typer.echo("Pushing changes to project Git remote")
         repo.git.push()
@@ -650,9 +738,8 @@ def get_status(
         overleaf_project_dir = os.path.join(
             ".calkit", "overleaf", overleaf_project_id
         )
-        overleaf_remote_url = (
-            f"https://git:{overleaf_token}@git.overleaf.com/"
-            f"{overleaf_project_id}"
+        overleaf_remote_url = calkit.overleaf.get_git_remote_url(
+            project_id=overleaf_project_id, token=str(overleaf_token)
         )
         if not os.path.isdir(overleaf_project_dir):
             overleaf_repo = git.Repo.clone_from(
